import os
import pkgutil
import webbrowser
from IPython.display import display, HTML
from typing import List, Optional, Union

from .LineageXNoConn import LineageXNoConn
from .LineageXWithConn import LineageXWithConn

IMPLEMENTED_DIALECTS_WITH_CONN = ("postgres",)


def save_js_file():
    for filename in ["app.js", "vendor.js"]:
        data = pkgutil.get_data(__name__, filename)

        with open(filename, "w", encoding="utf-8") as js_file:
            if data:
                js_file.write(data.decode("utf-8"))


def validate_sql(sql: Union[List, str]) -> None:
    if sql is None:
        raise ValueError(
            "The SQL input cannot be empty, please input a list of SQL or a path to SQL"
        )
    elif not isinstance(sql, list) and not isinstance(sql, str):
        raise ValueError(
            "Wrong SQL input format, please input a list of SQL or a path to SQL"
        )


def validate_schema(target_schema: str, search_path_schema: str) -> tuple:
    if target_schema == "" and search_path_schema == "":
        target_schema = "public"
        search_path_schema = "public"
    elif target_schema == "" and not search_path_schema == "":
        target_schema = search_path_schema.split(",")[0]
    elif not target_schema == "" and search_path_schema == "":
        search_path_schema = target_schema

    return target_schema, search_path_schema


class lineagex:
    def __init__(
        self,
        sql: Optional[Union[List, str]] = None,
        target_schema: Optional[str] = "",
        conn_string: Optional[str] = None,
        search_path_schema: Optional[str] = "",
        dialect: str = "postgres",
    ) -> None:
        validate_sql(sql)
        target_schema, search_path_schema = validate_schema(
            target_schema, search_path_schema
        )

        if conn_string:
            if dialect not in IMPLEMENTED_DIALECTS_WITH_CONN:
                raise NotImplemented(f"Not implemented dialect: {dialect}")

            lx = LineageXWithConn(
                sql=sql,
                target_schema=target_schema,
                conn_string=conn_string,
                search_path_schema=search_path_schema,
            )
            save_js_file()
            self.output_dict = lx.output_dict
        else:
            lx = LineageXNoConn(
                sql=sql,
                dialect=dialect,
                target_schema=target_schema,
                search_path_schema=search_path_schema,
            )
            save_js_file()
            self.output_dict = lx.output_dict

    def show(self):
        with open("index.html", "r", encoding="utf-8") as file_html:
            curr_html = file_html.read()
        with open("index_jupyter.html", "w", encoding="utf-8") as file_html:
            file_html.write(
                """<div style="width:100%; height:800px;">
                {}
            </div>""".format(
                    curr_html
                )
            )
        display(HTML("index_jupyter.html"))

    def show_tab(self):
        cwd = os.getcwd()
        p = os.path.join(cwd, "index.html").replace("\\", "/")
        print("opening the lineage page from {}".format(p))
        webbrowser.open_new_tab(f"file://{p}")


if __name__ == "__main__":
    pass



import re
from typing import Any, List, Optional, Tuple

from psycopg2.extensions import connection
from sqlglot import exp, parse_one
from sqlglot import expressions
from sqlglot.expressions import CTE

from .utils import find_column


class ColumnLineage:
    def __init__(
        self,
        plan: Optional[dict] = None,
        sql: Optional[str] = "",
        columns: Optional[List] = None,
        conn: Any = None,
        part_tables: Optional[dict] = None,
        search_schema: Optional[str] = "",
    ) -> None:
        self.split_regex = re.compile(r"[^a-zA-Z0-9._]")
        self.all_used_col = []
        self.possible_columns = []
        self.table_alias = {}
        self.table_alias_reversed = {}
        self.cte_name = ""
        self.agg_flag = False
        self.cte_dict = {}
        self.subplan_dict = {}
        self.function_call_cols = {}
        self.part_tables = part_tables
        self.column_prefix_dict = {}
        self.conn = conn
        self.search_schema = search_schema
        self.sql_ast = parse_one(sql=sql, read="postgres")
        self.cte_column = self._find_cte_col()
        self.final_output = ""
        self.final_node_type = ""
        self.subquery_final_output = ""
        self.column_dict = {}
        self.table_list = []
        self._traverse_plan(plan=plan)
        self._resolve_column_dict(cols=columns)
        self.table_list = sorted(set(self.table_list))
        # print(self.final_output)
        # print(self.subplan_dict)
        # print(self.table_alias)
        # print(self.cte_dict)
        # print(self.all_used_col)
        # print(self.possible_columns)
        # print("columns: ", self.column_dict)
        # print("table: ", self.table_list)

    def _resolve_column_dict(self, cols: Optional[List] = None) -> None:
        """
        Insert into column_dict using the column names as keys and its dependencies as child
        :param cols: the columns for the final output
        :return: None
        """
        if len(self.final_output) > len(cols):
            self.final_output = self.final_output[: len(cols)]
        elif len(self.final_output) < len(cols):
            print(
                "number of columns from the sql does not match the number in manifest"
            )
            return
        for idx, val in enumerate(self.final_output):
            # Postgres Aggregate function of count(*) that involves all the columns
            if self.final_node_type == "Aggregate" and val.strip() == "count(*)":
                all_cols = self.possible_columns
            else:
                table_col = re.split(self.split_regex, val.strip())
                all_cols = list(
                    set(set(table_col) & set(self.possible_columns)).union(
                        self.all_used_col
                    )
                )
            # Subquery in the final projection
            if val.find("SubPlan ") != -1:
                subplan_name = re.findall(re.compile(r"SubPlan [0-9]*"), val)[0]
                all_cols.extend(self.subplan_dict[subplan_name])
            self.column_dict[cols[idx]] = sorted(
                self._remove_table_alias(cols=all_cols)
            )

    def _traverse_plan(self, plan: Optional[dict] = None) -> None:
        """
        Traversing the plan using recursion and go into Plans if it is present
        :param plan: the given execution plan
        :return: None
        """
        if plan["Node Type"] in [
            "Seq Scan",
            "Parallel Seq Scan",
            "Bitmap Heap Scan",
            "Index Scan",
            "Index Only Scan",
        ]:
            self.table_alias[plan["Alias"]] = self._find_parent_table(
                table=plan["Schema"] + "." + plan["Relation Name"]
            )
            self.table_alias_reversed[
                plan["Schema"] + "." + plan["Relation Name"]
            ] = plan["Alias"]
        if "Plans" in plan.keys():
            for subplan_data in plan["Plans"]:
                self._traverse_plan(plan=subplan_data)
        temp = plan.get("Output")
        if temp is not None:
            self.final_output = temp
            self.final_node_type = plan.get("Node Type")
        # if scan from tables/views, add to possible columns
        if plan["Node Type"] in [
            "Seq Scan",
            "Parallel Seq Scan",
            "Bitmap Heap Scan",
            "Index Scan",
            "Index Only Scan",
        ]:
            # Scan and Filter and CTE in one
            if "Subplan Name" in plan.keys():
                self._add_possible_columns(plan)
                # self.possible_columns.extend(plan["Output"])
                if "Alias" in plan.keys():
                    alias = plan["Alias"]
                else:
                    alias = plan["Schema"] + "." + plan["Relation Name"]
                temp_regex = re.compile(r"({}\.[a-zA-Z0-9_]+)".format(alias))
                # print(plan['Subplan Name'], plan['Output'])
                for i in plan["Output"]:
                    self.possible_columns.extend(re.findall(temp_regex, i))
                if "Filter" in plan.keys():
                    self.possible_columns.extend(re.findall(temp_regex, plan["Filter"]))
                cte_name = plan["Subplan Name"].split(" ")[1]
                self.table_list.append(
                    self._find_parent_table(
                        table=(plan["Schema"] + "." + plan["Relation Name"])
                    )
                )
                self.table_alias[cte_name] = cte_name
                self._add_cte_dict(plan=plan)
            # Filter and scan in one plan
            elif "Filter" in plan.keys():
                self._add_possible_columns(plan)
                # self.possible_columns.extend(plan["Output"])
                self._handle_filter_in_scan(plan=plan)
                # in the filter, there is chance there is also index cond
                self._handle_index_cond(plan=plan)
            # Scan only
            else:
                # to avoid index cond
                self._add_possible_columns(plan=plan)
                self._handle_index_cond(plan=plan)
        # if scan from cte
        elif plan["Node Type"] == "CTE Scan":
            # add to possible column first and add all columns from CTE to prevent filters on top of any
            if "Alias" in plan.keys():
                all_cte_cols = [
                    plan["Alias"] + "." + s
                    for s in list(self.cte_dict[plan["CTE Name"]].keys())
                ]
            else:
                all_cte_cols = [
                    plan["CTE Name"] + "." + s
                    for s in list(self.cte_dict[plan["CTE Name"]].keys())
                ]
            self.possible_columns.extend(
                list(set(all_cte_cols).union(set(plan["Output"])))
            )
            # handle filter and scan in the same plan
            if "Filter" in plan.keys():
                self._handle_filter_in_scan(plan=plan)
            # the current scan can also be the creation of another cte
            if "Subplan Name" in plan.keys():
                if "CTE Name" in plan.keys():
                    self.possible_columns.extend(
                        [
                            plan["CTE Name"] + "." + s
                            for s in list(self.cte_dict[plan["CTE Name"]].keys())
                        ]
                    )
                self.table_alias[plan["Alias"]] = plan["CTE Name"]
                self._add_cte_dict(plan=plan)
            else:
                # if just a scan, add to alias
                self.table_alias[plan["Alias"]] = plan["CTE Name"]
        # if scan from a subquery
        elif plan["Node Type"] == "Subquery Scan":
            org_all_used_cols = self.all_used_col
            org_possible_cols = self.possible_columns
            self.all_used_col = []
            self.possible_columns = []
            if "Plans" in plan.keys():
                self.subquery_final_output = plan["Plans"][0]["Output"]
            else:
                self.subquery_final_output = plan["Output"]
            self._resolve_subquery(plan=plan)
            if "Alias" in plan.keys():
                self.cte_name = plan["Alias"]
            temp_dict = {}
            self._extract_from_cond(plan=plan)
            if self.cte_name in self.cte_column.keys():
                for idx, val in enumerate(self.cte_column[self.cte_name]):
                    cte_col = re.split(
                        self.split_regex, self.subquery_final_output[idx].strip()
                    )
                    all_cols = list(
                        set(set(cte_col) & set(self.possible_columns)).union(
                            self.all_used_col
                        )
                    )
                    # Subquery in the final projection
                    if self.subquery_final_output[idx].find("SubPlan ") != -1:
                        subplan_name = re.findall(
                            re.compile(r"SubPlan [0-9]*"),
                            self.subquery_final_output[idx],
                        )[0]
                        all_cols.extend(self.subplan_dict[subplan_name])
                    temp_dict[val] = self._remove_table_alias(cols=all_cols)
                self.cte_dict[self.cte_name] = temp_dict
            self.table_alias[plan["Alias"]] = plan["Alias"]
            self.all_used_col = org_all_used_cols
            self.possible_columns = org_possible_cols
            self.possible_columns.extend(plan["Output"])
        else:
            # creation of cte but with an Append node(usually UNION/EXCEPT/INTERSECT)
            if "Subplan Name" in plan.keys():
                # Resolve UNION/EXCEPT/INTERSECT since the Node Type will be Appended with no outputs
                if (
                    plan["Node Type"] in ["Append", "MergeAppend"]
                    and "Output" not in plan.keys()
                    and len(plan["Plans"]) != 0
                ):
                    s = plan.get("Subplan Name").split(" ")
                    if s[0] == "CTE":
                        self.cte_name = s[1]
                        temp_dict = {}
                        self._extract_from_cond(plan=plan)
                        self._resolve_union(plan=plan)
                        self.agg_flag = False
                        for val in self.cte_column[self.cte_name]:
                            temp_dict[val] = self._remove_table_alias(
                                cols=self.all_used_col
                            )
                        self.cte_dict[self.cte_name] = temp_dict
                        self.all_used_col = []
                        self.possible_columns = []
                else:
                    # if only creation of a cte, but no scan, just add it to cte_dict
                    self._add_cte_dict(plan=plan)
            # UNION/EXCEPT/INTERSECT at the last step with no creation of CTE
            elif (
                plan["Node Type"] in ["Append", "MergeAppend"]
                and "Output" not in plan.keys()
                and len(plan["Plans"]) != 0
            ):
                self._extract_from_cond(plan=plan)
                self._resolve_union(plan=plan)
                self.agg_flag = False
            # every other node aside from cte creations/scans
            else:
                self._extract_from_cond(plan=plan)

    def _handle_index_cond(self, plan: Optional[dict] = None) -> None:
        """
        When there's an index_cond in the plan, handle it and extract the necessary columns
        :param plan: the plan with the index_cond
        """
        temp = plan.get("Index Cond")
        if temp is not None:
            idx_name = plan.get("Index Name")
            if idx_name is not None:
                if isinstance(self.conn, connection):
                    # Postgres
                    cur = self.conn.cursor()
                    cur.execute("""SET search_path TO {};""".format(self.search_schema))
                    cur.execute(
                        "SELECT schemaname, tablename, indexname, indexdef FROM pg_indexes WHERE indexname = '{}'".format(
                            idx_name
                        )
                    )
                    result = cur.fetchall()[0]
                    cur.close()
                    # the indexdef is at index 3
                    indexdef = result[3]
                else:
                    # FalDbt
                    idx_fal = self.conn.execute_sql(
                        "SELECT schemaname, tablename, indexname, indexdef FROM pg_indexes WHERE indexname = '{}'".format(
                            idx_name
                        )
                    )
                    result = idx_fal.iloc[0]
                    indexdef = result["indexdef"]
                btree_idx = indexdef.find("USING btree")
                if btree_idx != -1:
                    close_bracket = indexdef.find(")", btree_idx)
                    if close_bracket != -1:
                        idx_cols = indexdef[btree_idx + 13 : close_bracket].split(",")
                        alias = self.table_alias_reversed[result[0] + "." + result[1]]
                        for i in idx_cols:
                            self.possible_columns.append(alias + "." + i)
            row_col = re.split(self.split_regex, temp.strip())
            self.all_used_col.extend(list(set(row_col) & set(self.possible_columns)))

    def _handle_filter_in_scan(self, plan: Optional[dict] = None) -> None:
        """
        When there is filter in the scan node, handle it and find necessary columns
        :param plan: the plan with the filter in the scan node
        """
        if "Alias" in plan.keys():
            alias = plan["Alias"]
        else:
            alias = plan["Schema"] + "." + plan["Relation Name"]
        temp_regex = re.compile(r"({}\.[a-zA-Z0-9_]+)".format(alias))
        self.possible_columns.extend(re.findall(temp_regex, plan["Filter"]))
        temp = plan.get("Filter")
        row_col = re.split(self.split_regex, temp.strip())
        self.all_used_col.extend(list(set(row_col) & set(self.possible_columns)))
        if temp.find("SubPlan ") != -1:
            subplan_name = re.findall(re.compile(r"SubPlan [0-9]*"), temp)[0]
            self.all_used_col.extend(self.subplan_dict[subplan_name])

    def _resolve_subquery(self, plan: Optional[dict] = None) -> None:
        """
        When there is a node with subquery, go in recursively and find the most inner node
        :param plan: the node with subquery
        """
        if "Plans" in plan.keys():
            for subplan_data in plan["Plans"]:
                self._resolve_subquery(plan=subplan_data)
        if "Output" in plan.keys() and plan["Node Type"] in [
            "Seq Scan",
            "Parallel Seq Scan",
            "Bitmap Heap Scan",
            "Index Scan",
            "Index Only Scan",
            "CTE Scan",
        ]:
            self.possible_columns.extend(plan["Output"])
            self._extract_from_cond(plan=plan)
        else:
            self._extract_from_cond(plan=plan)

    def _resolve_union(self, plan: Optional[dict] = None) -> None:
        """
        To resolve the UNION, all the columns involved are used since those columns need to be the same
        :param plan: the execution plan for UNION/EXCEPT/INTERSECT
        """
        if "Plans" in plan.keys():
            # Check if it is an aggregation, since it would scan all the columns
            if plan["Plans"][0]["Node Type"] == "Aggregate":
                self.agg_flag = True
            for subplan_data in plan["Plans"]:
                self._resolve_union(plan=subplan_data)
        if (
            "Output" in plan.keys()
            and plan["Node Type"]
            in [
                "Seq Scan",
                "Parallel Seq Scan",
                "Bitmap Heap Scan",
                "Index Scan",
                "Index Only Scan",
            ]
            and not self.agg_flag
        ):
            # check if it is using Append node for partitioned tables
            if plan["Schema"] + "." + plan["Relation Name"] in list(
                self.part_tables.keys()
            ):
                self.possible_columns.extend(plan["Output"])
                self._extract_from_cond(plan=plan)
            else:
                # if it is using Append node for UNION/EXCEPT/INTERSECT
                for col in plan["Output"]:
                    row_col = re.split(self.split_regex, col.strip())
                    self.all_used_col.extend(
                        list(set(row_col) & set(self.possible_columns))
                    )
        elif (
            "Output" in plan.keys()
            and plan["Node Type"] == "CTE Scan"
            and not self.agg_flag
        ):
            # if it is using Append node for UNION/EXCEPT/INTERSECT
            for col in plan["Output"]:
                row_col = re.split(self.split_regex, col.strip())
                self.all_used_col.extend(
                    list(set(row_col) & set(self.possible_columns))
                )

    def _add_cte_dict(self, plan: Optional[dict] = None) -> None:
        """
        Add to the cte dict given the CTE plan and analyze its column lineage
        :param plan: the CTE plan
        :return:
        """
        s = plan.get("Subplan Name").split(" ")
        if s[0] == "CTE":
            self.cte_name = s[1]
            temp_dict = {}
            self._extract_from_cond(plan=plan)
            for idx, val in enumerate(self.cte_column[self.cte_name]):
                cte_col = re.split(self.split_regex, plan["Output"][idx].strip())
                all_cols = list(
                    set(set(cte_col) & set(self.possible_columns)).union(
                        self.all_used_col
                    )
                )
                # Subquery in the final projection
                if plan["Output"][idx].find("SubPlan ") != -1:
                    subplan_name = re.findall(
                        re.compile(r"SubPlan [0-9]*"), plan["Output"][idx]
                    )[0]
                    all_cols.extend(self.subplan_dict[subplan_name])
                temp_dict[val] = self._remove_table_alias(cols=all_cols)
            self.cte_dict[self.cte_name] = temp_dict
            self.all_used_col = []
            self.possible_columns = []
        # temporary subplan name
        elif s[0] == "SubPlan":
            subplan_list = []
            for _, val in enumerate(plan["Output"]):
                table_col = re.split(self.split_regex, val.strip())
                all_cols = list(set(set(table_col) & set(self.possible_columns)))
                subplan_list.extend(self._remove_table_alias(cols=all_cols))
            self.subplan_dict[plan["Subplan Name"]] = subplan_list

    def _extract_from_cond(self, plan: Optional[dict] = None) -> None:
        """
        Extract column from multiple operators, some add to possible_cols and some add to all_used_cols
        :param plan: the execution plan for the operator
        :return:
        """
        # More conditions
        if plan["Node Type"] == "WindowAgg":
            self.possible_columns.extend(plan["Output"])
        # Handle index cond
        self._handle_index_cond(plan=plan)
        temp = plan.get("Hash Cond")
        if temp is not None:
            row_col = re.split(self.split_regex, temp.strip())
            self.all_used_col.extend(list(set(row_col) & set(self.possible_columns)))
        temp = plan.get("Merge Cond")
        if temp is not None:
            row_col = re.split(self.split_regex, temp.strip())
            self.all_used_col.extend(list(set(row_col) & set(self.possible_columns)))
        temp = plan.get("Recheck Cond")
        if temp is not None:
            row_col = re.split(self.split_regex, temp.strip())
            self.all_used_col.extend(list(set(row_col) & set(self.possible_columns)))
        temp = plan.get("Join Filter")
        if temp is not None:
            row_col = re.split(self.split_regex, temp.strip())
            self.all_used_col.extend(list(set(row_col) & set(self.possible_columns)))
        temp = plan.get("Filter")
        if temp is not None:
            row_col = re.split(self.split_regex, temp.strip())
            self.all_used_col.extend(list(set(row_col) & set(self.possible_columns)))
        temp = plan.get("Sort Key")
        if temp is not None:
            self.all_used_col.extend(list(set(temp) & set(self.possible_columns)))
        temp = plan.get("Group Key")
        if temp is not None:
            self.all_used_col.extend(list(set(temp) & set(self.possible_columns)))
        # Function calls
        if plan["Node Type"] == "Function Scan":
            if "Function Name" in plan.keys():
                if plan["Function Name"] == "unnest":
                    func_cols = re.split(
                        self.split_regex, plan["Function Call"].strip()
                    )
                    all_cols = list(
                        set(set(func_cols) & set(self.possible_columns)).union(
                            self.all_used_col
                        )
                    )
                    self.function_call_cols[plan["Output"][0]] = sorted(
                        self._remove_table_alias(cols=all_cols)
                    )
                    self.possible_columns.append(plan["Output"][0])

    def _add_possible_columns(self, plan: Optional[dict] = None) -> None:
        """
        Extract the columns/tables from the plan and add to possible columns and used tables, mainly used for base table
        :param plan: plan to extract columns and tables
        :return:
        """
        invalid_list = []
        for i in plan["Output"]:
            if len(i.split(".")) == 1:
                self.column_prefix_dict[i] = (
                    plan["Schema"] + "." + plan["Relation Name"] + "." + i
                )
        for i in plan["Output"]:
            # if a column name contains an invalid char(that is not a-zA-z0-9_)
            if re.search(r"[^\w.]", i):
                invalid_list.append(i)
                continue
            # if a column name starts with a digit, likely is just a number, but not a column
            elif i[0].isdigit():
                continue
            # no table prefix column outputs, append only the column name and the prefix + column
            elif len(i.split(".")) == 1:
                if "Alias" in plan.keys():
                    prefix = plan["Alias"]
                else:
                    prefix = plan["Schema"] + "." + plan["Relation Name"]
                self.possible_columns.append(prefix + "." + i)
                self.possible_columns.append(i)
            else:
                self.possible_columns.append(i)
        # if an output has invalid char, its likely it's an expression, have to extract the columns
        if invalid_list:
            all_table_cols = find_column(
                table_name=plan["Schema"] + "." + plan["Relation Name"],
                engine=self.conn,
                search_schema=self.search_schema,
            )
            for i in invalid_list:
                # split the output and match columns from all the columns of the table
                table_col = re.split(self.split_regex, i.strip())
                resolved_possible_cols = list(set(set(table_col) & set(all_table_cols)))
                for j in resolved_possible_cols:
                    self.possible_columns.append(j)
                    if len(j.split(".")) == 1:
                        self.column_prefix_dict[j] = (
                            plan["Schema"] + "." + plan["Relation Name"] + "." + j
                        )
        self.table_list.append(
            self._find_parent_table(table=plan["Schema"] + "." + plan["Relation Name"])
        )
        self.table_alias[plan["Alias"]] = self._find_parent_table(
            table=plan["Schema"] + "." + plan["Relation Name"]
        )
        self.table_alias_reversed[plan["Schema"] + "." + plan["Relation Name"]] = plan[
            "Alias"
        ]

    def _find_parent_table(self, table: Optional[str] = "") -> str:
        """
        Find the parent table from a given table name
        :param table: table name
        :return: table: the parent table name(if it is not partitioned table, it is just the input
        """
        if self.part_tables is not None:
            if table in self.part_tables.keys():
                table = self.part_tables[table]
        return table

    def _remove_table_alias(self, cols: Optional[List] = None) -> List:
        """
        Remove the alias in the name or add intended schema.table to column names
        :param cols: the list of columns that has aliases that need to be resolved
        :return: resolved aliases for the columns
        """
        current_cte_table = ""
        ret_cols = []
        temp_keys_dict = {}
        for i in cols:
            temp = i.split(".")
            # has prefix or no
            if len(temp) > 1:
                # prefix yes, is the prefix already in alias or base table name
                if temp[0] in self.table_alias.keys():
                    org_name = self.table_alias[temp[0]]
                    # prefix yes, get the non-alias cte name
                    if org_name in self.cte_dict.keys():
                        # to avoid case difference, create a temp dict for lower->original
                        # if the current cte's temp dict is already created, skip it
                        if org_name != current_cte_table:
                            cte_dict_keys = list(self.cte_dict[org_name].keys())
                            temp_keys_dict = {}
                            for k in cte_dict_keys:
                                temp_keys_dict[k.lower()] = k
                            current_cte_table = org_name
                        # find the matching column name in the cte_dict
                        if temp[1] in temp_keys_dict.keys():
                            ret_cols.extend(
                                self.cte_dict[org_name][temp_keys_dict[temp[1]]]
                            )
                    # prefix yes, get the non-alias base table name
                    else:
                        ret_cols.append(org_name + "." + temp[1])
                # prefix yes, is it from a function call temp col
                elif i in self.function_call_cols.keys():
                    ret_cols.extend(self.function_call_cols[i])
                else:
                    ret_cols.append(i)
            else:
                ret_cols.append(self.column_prefix_dict[i])
        return list(set(ret_cols))

    def _find_table(self, cte: CTE = None) -> Tuple[dict, List]:
        """
        Find aliases for the tables in the cte
        :param cte: the sql of the sql to be analyzed
        :return: the dict with table name as key and alias as child
        """
        table_alias_dict = {}
        for table in cte.find_all(exp.Table):
            table_def_split = re.split(" AS ", table.sql(), flags=re.IGNORECASE)
            if len(table_def_split) == 1:
                table_alias_dict[table_def_split[0]] = table_def_split[0]
            else:
                table_alias_dict[table_def_split[1]] = table_def_split[0]
        # Find tables that's only in the CTE but not in the Subquery
        temp_cte = cte.copy()
        for sub_ast in temp_cte.find_all(exp.Subquery):
            sub_ast.pop()
        cte_table_list = []
        for table in temp_cte.find_all(exp.Table):
            table_def_split = re.split(" AS ", table.sql(), flags=re.IGNORECASE)
            cte_table_list.append(table_def_split[0])
        return table_alias_dict, cte_table_list

    def _find_cte_col(self) -> dict:
        """
        Find the column names for each cte since it does not show in the execution plan
        :param sql: the sql that needs to be analyzed
        :return: the dict with table name as keys and list of columns as child
        """
        cte_col_dict = {}
        for cte in self.sql_ast.find_all(exp.CTE):
            cte_col_dict = self._find_cte_col_func(cte=cte, cte_col_dict=cte_col_dict)
        for cte in self.sql_ast.find_all(exp.Subquery):
            cte_col_dict = self._find_cte_col_func(cte=cte, cte_col_dict=cte_col_dict)
        return cte_col_dict

    def _find_cte_col_func(
        self, cte: expressions = None, cte_col_dict: Optional[dict] = None
    ) -> dict:
        """
        The function to find all the column names for the cte
        :param cte: the cte ast tree
        :param cte_col_dict: the dict that stores the column names for the previous cte
        :return: the dict that contains column names with the current cte
        """
        # Find each CTE
        cte_name = cte.find(exp.TableAlias).alias_or_name
        cte_col_dict[cte_name] = []
        # Iterate column for each CTE
        if cte.find(exp.Select):
            for projection in cte.find(exp.Select).expressions:
                col_name = projection.alias_or_name
                # Resolve aggregations with no alias
                if isinstance(projection, exp.Count):
                    col_name = "count"
                elif isinstance(projection, exp.Avg):
                    col_name = "avg"
                elif isinstance(projection, exp.Max):
                    col_name = "max"
                elif isinstance(projection, exp.Min):
                    col_name = "min"
                elif isinstance(projection, exp.Sum):
                    col_name = "sum"
                # Resolve * and check if it is from a previous CTE or check database for the table
                elif (
                    isinstance(projection, exp.Column) and projection.find(exp.Star)
                ) or col_name == "*":
                    table_alias_dict, cte_table_list = self._find_table(cte=cte)
                    # if the * has a prefix
                    if projection.find(exp.Identifier):
                        t_name = projection.find(exp.Identifier).text("this")
                        if table_alias_dict[t_name] in cte_col_dict.keys():
                            col_name = cte_col_dict[table_alias_dict[t_name]]
                        else:
                            col_name = find_column(
                                table_name=table_alias_dict[t_name],
                                engine=self.conn,
                                search_schema=self.search_schema,
                            )
                    # if * has no prefix
                    else:
                        for t_name in cte_table_list:
                            if t_name in cte_col_dict.keys():
                                cte_col_dict[cte_name].extend(cte_col_dict[t_name])
                            else:
                                cte_col_dict[cte_name].extend(
                                    find_column(
                                        table_name=t_name,
                                        engine=self.conn,
                                        search_schema=self.search_schema,
                                    )
                                )
                if isinstance(col_name, list):
                    cte_col_dict[cte_name].extend(col_name)
                else:
                    if col_name != "*":
                        cte_col_dict[cte_name].append(col_name)
        return cte_col_dict


if __name__ == "__main__":
    pass



from typing import List, Optional, Tuple, Union
import time

import psycopg2
from psycopg2 import OperationalError
from psycopg2.extensions import connection

from .ColumnLineage import ColumnLineage
from .SqlToDict import SqlToDict
from .stack import *
from .utils import find_column, produce_json


class LineageXWithConn:
    def __init__(
        self,
        sql: Optional[Union[List, str]] = None,
        target_schema: Optional[str] = "public",
        conn_string: Optional[str] = "",
        search_path_schema: Optional[str] = "public",
    ) -> None:
        self.parsed = 0
        self.not_parsed = 0
        self.transaction_time = 0
        self.part_tables = None
        self.df = None
        self.schema = target_schema
        self.search_schema = target_schema + "," + search_path_schema
        self.new_view_list = []
        self.s = Stack()
        self.sql = sql
        self.sql_files_dict = {}
        self.creation_list = []
        self.finished_list = []
        self.output_dict = {}
        self.conn = self._check_db_connection(conn_string)
        self.conn.autocommit = True
        self._run_table_lineage()

    def _run_table_lineage(self) -> None:
        """
        The driver function to extract the table lineage information
        :return: output an interactive html for the table lineage information
        """
        start_time = time.time()
        self.part_tables, self.schema_list = self._get_part_tables()
        # If the input is a list with no SELECT , assume it to be a list of views/schema
        if isinstance(self.sql, List) and not any(
            "SELECT " in s.upper() for s in self.sql
        ):
            cur = self.conn.cursor()
            cur.execute("""SET search_path TO {};""".format(self.search_schema))
            for t in self.sql:
                temp = t.split(".")
                if len(temp) == 2:
                    cur.execute(
                        "SELECT concat_ws('.',schemaname,viewname) AS view_name, definition FROM pg_catalog.pg_views WHERE schemaname = '{}' and viewname = '{}';".format(
                            temp[0], temp[1]
                        )
                    )
                    view_ret = cur.fetchall()
                    if view_ret:
                        self.sql_files_dict[view_ret[0][0]] = view_ret[0][1]
                    else:
                        print(
                            "{} is skipped because database returned no result on it".format(
                                t
                            )
                        )
                elif len(temp) == 1:
                    cur.execute(
                        "SELECT concat_ws('.',schemaname,viewname) AS view_name, definition FROM pg_catalog.pg_views WHERE schemaname = '{}';".format(
                            temp[0]
                        )
                    )
                    schema_ret = cur.fetchall()
                    if schema_ret:
                        for s in schema_ret:
                            self.sql_files_dict[s[0]] = s[1]
                    else:
                        print(
                            "{} is skipped because database returned no result on it".format(
                                t
                            )
                        )
            cur.close()
            for name, sql in self.sql_files_dict.items():
                try:
                    print(name, " processing")
                    col_lineage = ColumnLineage(
                        plan=self._get_plan(sql=sql),
                        sql=sql,
                        columns=find_column(
                            table_name=name,
                            engine=self.conn,
                            search_schema=self.search_schema,
                        ),
                        conn=self.conn,
                        part_tables=self.part_tables,
                        search_schema=self.search_schema,
                    )
                    self.output_dict[name] = {
                        "tables": col_lineage.table_list,
                        "columns": col_lineage.column_dict,
                        "table_name": name,
                        "sql": sql,
                    }
                    self.parsed += 1
                except Exception as e:
                    print("{} is not processed because it countered {}".format(name, e))
                    self.not_parsed += 1
                    continue
        # path or a list of SQL that at least one element contains
        else:
            self.sql_files_dict = SqlToDict(self.sql, self.schema_list).sql_files_dict
            for name, sql in self.sql_files_dict.items():
                try:
                    if name not in self.finished_list:
                        self._explain_sql(name=name, sql=sql)
                    else:
                        continue
                except Exception as e:
                    print("{} is not processed because it countered {}".format(name, e))
                    self.not_parsed += 1
                    continue
        produce_json(
            output_dict=self.output_dict,
            engine=self.conn,
            search_schema=self.search_schema,
        )
        self._delete_view()
        self.conn.close()
        print(
            "{} SQLs are parsed, {} SQLs are not parsed, took a total of {:.1f} seconds, total transaction time is {:.1f}".format(
                self.parsed,
                self.not_parsed,
                time.time() - start_time,
                self.transaction_time,
            )
        )
        # print("total transaction time: ", self.transaction_time)

    def _delete_view(self) -> None:
        """
        Delete all temporary tables in the new_view_list
        :return: None
        """
        # reverse it just in case to drop dependencies first
        self.new_view_list = self.new_view_list[::-1]
        start_time = time.time()
        cur = self.conn.cursor()
        for i in self.new_view_list:
            cur.execute("""DROP TABLE {} CASCADE""".format(i))
            print(i + " dropped")
        cur.close()
        self.transaction_time += time.time() - start_time

    def _create_view(self, name: Optional[str] = "", sql: Optional[str] = "") -> None:
        """
        Create temporary tables with no data from the given sql
        :param name: name of the table
        :param sql: the sql for the table
        :param conn: the psycopg2 connection it was using
        :return: None
        """
        # connect and create view
        start_time = time.time()
        cur = self.conn.cursor()
        cur.execute(
            """SET search_path TO {}, {};""".format(self.search_schema, self.schema)
        )
        if sql.endswith(";"):
            cur.execute(
                """CREATE TABLE {}.{} AS {} WITH NO DATA;""".format(
                    self.schema, name, sql[:-1]
                )
            )
        else:
            cur.execute(
                """CREATE TABLE {}.{} AS {} WITH NO DATA;""".format(
                    self.schema, name, sql
                )
            )
        cur.close()
        self.transaction_time += time.time() - start_time
        print(self.schema + "." + name + " created")

    def _get_plan(self, sql: Optional[str] = "") -> dict:
        """
        Get the plan by providing the sql
        :param sql: the sql for getting the plan
        :return: the physical plan of the sql
        """
        start_time = time.time()
        cur = self.conn.cursor()
        cur.execute("""SET search_path TO {};""".format(self.search_schema))
        cur.execute(
            """EXPLAIN (VERBOSE TRUE, FORMAT JSON, COSTS FALSE) {}""".format(sql)
        )
        log_plan = cur.fetchall()
        cur.close()
        self.transaction_time += time.time() - start_time
        while True:
            if isinstance(log_plan, list) or isinstance(log_plan, tuple):
                log_plan = log_plan[0]
            elif isinstance(log_plan, dict):
                log_plan = log_plan["Plan"]
                break
        return log_plan

    def _explain_sql(self, name: Optional[str] = "", sql: Optional[str] = "") -> None:
        """
        Main function for extracting the table name from the sql. It tries to explain the current file's sql by
        analyzing the logical plan. But if its dependency is missing, the current one is put onto a stack and checking
        on the dependency table, then pops the stack when the current one is done.
        :param name: name of the file
        :param sql: the sql from the file
        :return: updates file_list, sql_list, table_list, new_view_list
        """
        try:
            print(name + " processing")
            if name in self.creation_list:
                self._create_view(name=name, sql=sql)
                self.new_view_list.append(self.schema + "." + name)
                self.creation_list.pop(self.creation_list.index(name))
                table_name = self.schema + "." + name
            elif name.isnumeric():
                table_name = "lineagex_temp_{}".format(name)
                self._create_view(name=table_name, sql=sql)
                self.new_view_list.append(self.schema + "." + table_name)
            elif name.find("_DELETION_") != -1 or name.find("_INSERTION_") != -1:
                table_name = name.replace(".", "_")
                self._create_view(name=table_name, sql=sql)
                self.new_view_list.append(self.schema + "." + table_name)
            else:
                cur = self.conn.cursor()
                cur.execute("""SET search_path TO {};""".format(self.search_schema))
                cur.execute(
                    """SELECT CONCAT (schemaname,'.', tablename) from pg_tables WHERE schemaname = '{0}' and tablename = '{1}'""".format(
                        self.schema, name
                    )
                )
                table_result = cur.fetchone()
                cur.close()
                if not table_result:
                    self._create_view(name=name, sql=sql)
                    self.new_view_list.append(self.schema + "." + name)
                table_name = self.schema + "." + name
            cols = find_column(
                table_name=table_name,
                engine=self.conn,
                search_schema=self.search_schema,
            )
            col_lineage = ColumnLineage(
                plan=self._get_plan(sql=sql),
                sql=sql,
                columns=cols,
                conn=self.conn,
                part_tables=self.part_tables,
                search_schema=self.search_schema,
            )
            if (
                name.isnumeric()
                or name.find("_DELETION_") != -1
                or name.find("_INSERTION_") != -1
            ):
                table_name = name
            self.output_dict[table_name] = {
                "tables": col_lineage.table_list,
                "columns": col_lineage.column_dict,
                "table_name": table_name,
                "sql": sql,
            }
            self.finished_list.append(name)
            self.parsed += 1
            while not self.s.isEmpty():
                if self.s.peek() in self.sql_files_dict.keys():
                    sql = self.sql_files_dict[self.s.peek()]
                    return self._explain_sql(name=self.s.pop(), sql=sql)
                else:
                    print("no sql file for the dependencies")
            return
        except psycopg2.ProgrammingError as e:
            # does not exist error code
            if e.pgcode == "42P01":
                error_msg = e.pgerror
                no_find_idx = error_msg.find("does not exist")
                relation_idx = error_msg.find("relation")
                schema_table = error_msg[relation_idx:no_find_idx]
                table_name = schema_table.split(" ")[-2].split(".")[-1].strip('"')
                self.s.push(name)
                # print(table_name, self.sql_files_dict)
                if table_name in self.sql_files_dict.keys():
                    if self.schema + "." + table_name in self.new_view_list:
                        print(
                            "{}.{} is already created, but the created schema is different from the queried schema for {} in {}.sql".format(
                                self.schema, table_name, table_name, name
                            )
                        )
                        return
                    print(
                        name
                        + " is dependant on "
                        + table_name
                        + ", creating that first\n"
                    )
                    self.creation_list.append(table_name)
                    sql = self.sql_files_dict[table_name]
                    return self._explain_sql(name=table_name, sql=sql)
                else:
                    print(
                        name
                        + " is skipped because it is missing dependency table "
                        + table_name
                    )
                    self.not_parsed += 1
                    return
            else:
                print(e)
                self.not_parsed += 1
        except Exception as e:
            print(e)
            self.not_parsed += 1

    def _check_db_connection(self, conn_string: Optional[str] = "") -> connection:
        """
        Check if the conn_string is good
        :return: the sqlalchemy engine
        """
        try:
            psycopg2.connect(conn_string)
            print("database connected")
        except OperationalError:
            print("authentication error")
        return psycopg2.connect(conn_string)

    def _get_part_tables(self) -> Tuple[dict, List]:
        """
        Find out the partition table in the Postgres database
        :return: the result with the parent and partitioned table
        """
        cur = self.conn.cursor()
        cur.execute(
            """SELECT
                concat_ws('.', nmsp_parent.nspname, parent.relname) AS parent,
                concat_ws('.', nmsp_child.nspname, child.relname) AS child
            FROM pg_inherits
                JOIN pg_class parent            ON pg_inherits.inhparent = parent.oid
                JOIN pg_class child             ON pg_inherits.inhrelid   = child.oid
                JOIN pg_namespace nmsp_parent   ON nmsp_parent.oid  = parent.relnamespace
                JOIN pg_namespace nmsp_child    ON nmsp_child.oid   = child.relnamespace"""
        )
        ret = cur.fetchall()
        parent_dict = {}
        for i in ret:
            parent_dict[i[1]] = i[0]
        # get the schema list as well
        cur.execute("select nspname from pg_catalog.pg_namespace;")
        result = cur.fetchall()
        schema_list = [s[0] for s in result]
        cur.close()
        return parent_dict, schema_list


if __name__ == "__main__":
    pass



import os
import re
from typing import List, Optional, Union

from .utils import find_select, get_files, remove_comments

rem_regex = re.compile(r"[^a-zA-Z0-9_.]")


class SqlToDict:
    def __init__(
        self, path: Optional[Union[List, str]] = "", schema_list: Optional[List] = None
    ) -> None:
        self.path = path
        self.schema_list = schema_list
        self.sql_files = []
        self.sql_files_dict = {}
        self.org_sql_files_dict = {}
        self.deletion_dict = {}
        self.insertion_dict = {}
        self.curr_name = ""
        self._sql_to_dict()
        pass

    def _sql_to_dict(self) -> None:
        """
        The driver function to make the input into the dict format, name of sql:sql
        :return:
        """
        if isinstance(self.path, list):
            for idx, val in enumerate(self.path):
                self._preprocess_sql(new_sql=val, file=str(idx), org_sql=val)
        else:
            self.sql_files = get_files(path=self.path)
            for f in self.sql_files:
                org_sql = open(f, mode="r", encoding="utf-8-sig").read()
                new_sql = remove_comments(str1=org_sql)
                org_sql_split = list(filter(None, new_sql.split(";")))
                # pop DROP IF EXISTS
                if len(org_sql_split) > 0:
                    for s in org_sql_split:
                        temp_str = s.upper()
                        if temp_str.find("SELECT ") == -1 and (
                            temp_str.startswith("DROP TABLE IF EXISTS")
                            or temp_str.startswith("DROP VIEW IF EXISTS")
                        ):
                            org_sql_split.pop(org_sql_split.index(s))
                if f.endswith(".sql") or f.endswith(".SQL"):
                    f = os.path.basename(f)[:-4]
                if len(org_sql_split) <= 1:
                    self._preprocess_sql(new_sql=org_sql_split[0], file=f, org_sql=org_sql)
                else:
                    for idx, val in enumerate(org_sql_split):
                        self._preprocess_sql(new_sql=val, file=f + "_" + str(idx), org_sql=org_sql)
        for key, value in self.sql_files_dict.copy().items():
            if key.startswith("."):
                self.sql_files_dict[key[1:]] = value
                del self.sql_files_dict[key]
        #print(self.sql_files_dict)

    def _preprocess_sql(
        self, new_sql: Optional[str] = "", file: Optional[str] = "", org_sql: Optional[str] = ""
    ) -> None:
        """
        Process the sql, remove database name in the clause/datetime_add/datetime_sub adding quotes
        :param new_sql: the sql for parsing, file: file name for the sql, org_sql: the most original sql
        :return: None
        """
        ret_sql = remove_comments(str1=new_sql)
        ret_sql = ret_sql.replace("`", "")
        # remove any database names in the query
        if self.schema_list:
            for i in self.schema_list:
                ret_sql = re.sub("[^ (,]*(\.{}\.)".format(i), "{}.".format(i), ret_sql)
        ret_sql = re.sub(
            r"DATETIME_DIFF\((.+?),\s?(.+?),\s?(DAY|MINUTE|SECOND|HOUR|YEAR)\)",
            r"DATETIME_DIFF(\1, \2, '\3'::TEXT)",
            ret_sql,
        )
        ret_sql = re.sub("datetime_add", "DATETIME_ADD", ret_sql, flags=re.IGNORECASE)
        ret_sql = re.sub("datetime_sub", "DATETIME_SUB", ret_sql, flags=re.IGNORECASE)
        # DATETIME_ADD '' value
        dateime_groups = re.findall(
            r"DATETIME_ADD\(\s?(.+?),\s?INTERVAL\s?(.+?)\s?(DAY|MINUTE|SECOND|HOUR|YEAR)\)",
            ret_sql,
        )
        if dateime_groups:
            for i in dateime_groups:
                if not i[1].startswith("'") and not i[1].endswith("'"):
                    ret_sql = ret_sql.replace(
                        "DATETIME_ADD({},INTERVAL {} {})".format(i[0], i[1], i[2]),
                        "DATETIME_ADD({},INTERVAL '{}' {})".format(i[0], i[1], i[2]),
                    )
                else:
                    continue
        # DATETIME_SUB '' value
        dateime_sub_groups = re.findall(
            r"DATETIME_SUB\(\s?(.+?),\s?INTERVAL\s?(.+?)\s?(DAY|MINUTE|SECOND|HOUR|YEAR)\)",
            ret_sql,
        )
        if dateime_sub_groups:
            for i in dateime_sub_groups:
                if not i[1].startswith("'") and not i[1].endswith("'"):
                    ret_sql = ret_sql.replace(
                        "DATETIME_SUB({},INTERVAL {} {})".format(i[0], i[1], i[2]),
                        "DATETIME_SUB({},INTERVAL '{}' {})".format(i[0], i[1], i[2]),
                    )
                else:
                    continue
        if re.search(
            "CREATE VIEW IF NOT EXISTS", ret_sql, flags=re.IGNORECASE
        ) or re.search("CREATE TABLE IF NOT EXISTS", ret_sql, flags=re.IGNORECASE):
            if ret_sql.upper().find("SELECT ") != -1:
                if bool(re.match('CREATE VIEW IF NOT EXISTS', ret_sql, re.I)) or bool(re.match('CREATE TABLE IF NOT EXISTS', ret_sql, re.I)):
                    temp = ret_sql.split(" ")
                    ret_sql = ret_sql[ret_sql.index(temp[7]):]
                else:
                    c_idx = ret_sql.upper().find("CREATE VIEW IF NOT EXISTS")
                    if c_idx == -1:
                        c_idx = ret_sql.upper().find("CREATE TABLE IF NOT EXISTS")
                    sub = ret_sql[c_idx:]
                    temp = sub.split(" ")
                    ret_sql = ret_sql[:c_idx] + " " + sub[sub.index(temp[7]):]
                name = temp[5]
                if name in self.sql_files_dict.keys():
                    print("WARNING: duplicate script detected for {}".format(name))
                self.sql_files_dict[name] = ret_sql
                self.org_sql_files_dict[name] = org_sql

        elif re.search("CREATE VIEW", ret_sql, flags=re.IGNORECASE) or re.search(
            "CREATE TABLE", ret_sql, flags=re.IGNORECASE
        ):
            if ret_sql.upper().find("SELECT ") != -1:
                if bool(re.match('CREATE VIEW', ret_sql, re.I)) or bool(re.match('CREATE TABLE', ret_sql, re.I)):
                    temp = ret_sql.split(" ")
                    ret_sql = ret_sql[ret_sql.index(temp[4]):]
                else:
                    c_idx = ret_sql.upper().find("CREATE VIEW")
                    if c_idx == -1:
                        c_idx = ret_sql.upper().find("CREATE TABLE")
                    sub = ret_sql[c_idx:]
                    temp = sub.split(" ")
                    ret_sql = ret_sql[:c_idx] + " " + sub[sub.index(temp[4]):]
                name = temp[2]
                if name in self.sql_files_dict.keys():
                    print("WARNING: duplicate script detected for {}".format(name))
                self.sql_files_dict[name] = ret_sql
                self.org_sql_files_dict[name] = org_sql

        # adjust to INSERT/DELETE/SELECT/
        elif ret_sql.find("INSERT INTO") != -1:
            # find the current name in the insertion dict and how many times it has been inserted
            self.curr_name = re.sub(rem_regex, "", ret_sql.split(" ")[2])
            if self.curr_name not in self.insertion_dict.keys():
                self.insertion_dict[self.curr_name] = 1
            else:
                self.insertion_dict[self.curr_name] = (
                    self.insertion_dict[self.curr_name] + 1
                )
            insert_counter = self.insertion_dict[self.curr_name]
            self.curr_name = self.curr_name + "_INSERTION_{}".format(insert_counter)
            self.sql_files_dict[self.curr_name] = find_select(q=ret_sql)
            self.org_sql_files_dict[self.curr_name] = org_sql
        elif ret_sql.find("DELETE FROM") != -1:
            # find the current name in the insertion dict and how many times it has been deleted
            self.curr_name = re.sub(rem_regex, "", ret_sql.split(" ")[2])
            if self.curr_name not in self.deletion_dict.keys():
                self.deletion_dict[self.curr_name] = 1
            else:
                self.deletion_dict[self.curr_name] = (
                    self.deletion_dict[self.curr_name] + 1
                )
            delete_counter = self.deletion_dict[self.curr_name]
            self.curr_name = self.curr_name + "_DELETION_{}".format(delete_counter)
            self.sql_files_dict[self.curr_name] = find_select(q=ret_sql)
            self.org_sql_files_dict[self.curr_name] = org_sql
        elif re.search("CREATE EXTENSION", ret_sql, flags=re.IGNORECASE):
            return
        else:
            if os.path.isfile(file):
                name = os.path.basename(file)[:-4]
                if name in self.sql_files_dict.keys():
                    print("WARNING: duplicate script detected for {}".format(name))
                self.sql_files_dict[name] = ret_sql
                self.org_sql_files_dict[name] = org_sql
            else:
                self.sql_files_dict[file] = ret_sql
                self.org_sql_files_dict[file] = org_sql


if __name__ == "__main__":
    pass



import os

from .lineagex import lineagex


class example:
    def __init__(self, case: str = ""):
        def get_current_path():
            # Returns the path of the current script
            return os.path.dirname(os.path.realpath(__file__))

        cwd = get_current_path()
        if case == "github_example":
            t_noconn = lineagex(
                sql=os.path.join(cwd, "examples", "github_example"),
                target_schema="schema1",
                search_path_schema="schema1, public",
            )
            print(
                "{} finished, please check in the folder for output.json and index.html".format(
                    case
                )
            )
        elif case == "dependency_example":
            t_noconn = lineagex(
                sql=os.path.join(cwd, "examples", "dependency_example"),
                target_schema="mimiciii_derived",
                search_path_schema="mimiciii_clinical, public",
            )
            print(
                "{} finished, please check in the folder for output.json and index.html".format(
                    case
                )
            )
        elif case == "mimic-iv":
            t_noconn = lineagex(
                sql=os.path.join(cwd, "examples", "mimic-iv"),
                target_schema="mimiciv_derived",
                search_path_schema="mimiciv_icu, mimiciv_hosp",
            )
            print(
                "{} finished, please check in the folder for output.json and index.html".format(
                    case
                )
            )
        elif case == "mimic-iii":
            t_noconn = lineagex(
                sql=os.path.join(cwd, "examples", "mimic-iii"),
                target_schema="mimiciii_derived",
                search_path_schema="mimiciii_clinical, public",
            )
            print(
                "{} finished, please check in the folder for output.json and index.html".format(
                    case
                )
            )


if __name__ == "__main__":
    example("mimic-iii")
